@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -459,7 +459,7 @@ static int scan_enable(uint8_t *data)
 
 	baswap(&ba, &vdev.bdaddr);
 	sa.sin_family = AF_INET;
-	sa.sin_addr.s_addr = *(uint32_t *) &ba;
+	memcpy(&sa.sin_addr.s_addr, &ba, sizeof(sa.sin_addr.s_addr));
 	sa.sin_port = *(uint16_t *) &ba.b[4];
 	if (bind(sk, (struct sockaddr *) &sa, sizeof(sa))) {
 		syslog(LOG_ERR, "Can't bind socket: %s (%d)",
@@ -518,7 +518,7 @@ static void disconnect(uint8_t *data)
 
 	handle = btohs(cp->handle);
 
-	if (handle - 1 > VHCI_MAX_CONN)
+	if (handle > VHCI_MAX_CONN)
 		return;
 
 	if (!(conn = vconn[handle-1]))
@@ -566,7 +566,7 @@ do_connect:
 
 	baswap(&ba, &cp->bdaddr);
 	sa.sin_family = AF_INET;
-	sa.sin_addr.s_addr = *(uint32_t *) &ba;
+	memcpy(&sa.sin_addr.s_addr, &ba, sizeof(sa.sin_addr.s_addr));
 	sa.sin_port = *(uint16_t *) &ba.b[4];
 	if (connect(sk, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
 		syslog(LOG_ERR, "Can't connect: %s (%d)",
@@ -603,7 +603,7 @@ do_connect:
 	return;
 }
 
-static void inline hci_link_control(uint16_t ocf, int plen, uint8_t *data)
+static void hci_link_control(uint16_t ocf, int plen, uint8_t *data)
 {
 	uint8_t status;
 
@@ -632,7 +632,7 @@ static void inline hci_link_control(uint16_t ocf, int plen, uint8_t *data)
 	}
 }
 
-static void inline hci_link_policy(uint16_t ocf, int plen, uint8_t *data)
+static void hci_link_policy(uint16_t ocf, int plen, uint8_t *data)
 {
 	uint8_t status;
 
@@ -646,7 +646,7 @@ static void inline hci_link_policy(uint16_t ocf, int plen, uint8_t *data)
 	}
 }
 
-static void inline hci_host_control(uint16_t ocf, int plen, uint8_t *data)
+static void hci_host_control(uint16_t ocf, int plen, uint8_t *data)
 {
 	read_local_name_rp ln;
 	read_class_of_dev_rp cd;
@@ -745,7 +745,7 @@ static void inline hci_host_control(uint16_t ocf, int plen, uint8_t *data)
 	}
 }
 
-static void inline hci_info_param(uint16_t ocf, int plen, uint8_t *data)
+static void hci_info_param(uint16_t ocf, int plen, uint8_t *data)
 {
 	read_local_version_rp lv;
 	read_local_features_rp lf;
@@ -762,7 +762,7 @@ static void inline hci_info_param(uint16_t ocf, int plen, uint8_t *data)
 		lv.hci_ver = 0x03;
 		lv.hci_rev = htobs(0x0000);
 		lv.lmp_ver = 0x03;
-		lv.err = htobs(29);
+		lv.manufacturer = htobs(29);
 		lv.lmp_subver = htobs(0x0000);
 		command_complete(ogf, ocf, sizeof(lv), &lv);
 		break;
@@ -1172,7 +1172,13 @@ static void usage(void)
 {
 	printf("hciemu - HCI emulator ver %s\n", VERSION);
 	printf("Usage: \n");
-	printf("\thciemu [-n] local_address\n");
+	printf("\thciemu [options] local_address\n"
+		"Options:\n"
+		"\t[-d device] use specified device\n"
+		"\t[-b bdaddr] emulate specified address\n"
+		"\t[-s file] create snoop file\n"
+		"\t[-n] do not detach\n"
+		"\t[-h] help, you are looking at it\n");
 }
 
 static struct option main_options[] = {
@@ -1314,7 +1320,8 @@ int main(int argc, char *argv[])
 	vdev.features[7] = 0x80;
 
 	memset(vdev.name, 0, sizeof(vdev.name));
-	strncpy((char *) vdev.name, "BlueZ (Virtual HCI)", sizeof(vdev.name));
+	strncpy((char *) vdev.name, "BlueZ (Virtual HCI)",
+							sizeof(vdev.name) - 1);
 
 	vdev.dev_class[0] = 0x00;
 	vdev.dev_class[1] = 0x00;
